package com.fjhx.purchase.service.subscribe.impl;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.entity.product.vo.ProductInfoVo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.purchase.entity.subscribe.dto.SubscribeDto;
import com.fjhx.purchase.entity.subscribe.dto.SubscribeSelectDto;
import com.fjhx.purchase.entity.subscribe.po.Subscribe;
import com.fjhx.purchase.entity.subscribe.po.SubscribeDetail;
import com.fjhx.purchase.entity.subscribe.vo.SubscribeVo;
import com.fjhx.purchase.mapper.subscribe.SubscribeMapper;
import com.fjhx.purchase.service.subscribe.SubscribeDetailService;
import com.fjhx.purchase.service.subscribe.SubscribeService;
import com.fjhx.purchase.util.code.CodeEnum;
import com.fjhx.wms.entity.warehouse.po.Warehouse;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysDeptService;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 申购单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-03
 */
@DS(SourceConstant.PURCHASE)
@Service
public class SubscribeServiceImpl extends ServiceImpl<SubscribeMapper, Subscribe> implements SubscribeService {

    @Autowired
    private SubscribeDetailService subscribeDetailService;

    @Autowired
    private ProductInfoService productInfoService;

    @Autowired
    WarehouseService warehouseService;
    @Autowired
    ISysDeptService deptService;

    @Autowired
    private CodingRuleService codingRuleService;


    @Override
    public Page<SubscribeVo> getPage(SubscribeSelectDto dto) {
        IWrapper<Subscribe> wrapper = getWrapper();
        wrapper.orderByDesc("s", Subscribe::getId);
        Page<SubscribeVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @Override
    public Subscribe detail(Long id) {
        if (ObjectUtils.isEmpty(id)) {
            throw new ServiceException("参数异常");
        }
        Subscribe subscribe = this.getById(id);
        if (ObjectUtils.isEmpty(subscribe)) {
            throw new ServiceException("数据不存在");
        }
        //处理申购产品
        List<SubscribeDetail> subscribeDetailList = subscribeDetailService.list(
                Wrappers.<SubscribeDetail>query().lambda()
                        .eq(SubscribeDetail::getSubscribeId, id));
        if (CollectionUtils.isNotEmpty(subscribeDetailList)) {
            //根据申购businessId取出所有产品
            List<Long> productIds = subscribeDetailList.stream().map(SubscribeDetail::getBussinessId).collect(Collectors.toList());
            List<ProductInfoVo> productInfoVos = productInfoService.getListByProductIds(productIds);
            Map<Long, List<ProductInfoVo>> productMap = productInfoVos.stream().collect(Collectors.groupingBy(ProductInfoVo::getId));
            for (SubscribeDetail s : subscribeDetailList) {
                if (MapUtils.isNotEmpty(productMap)) {
                    ProductInfoVo p = productMap.get(s.getBussinessId()).get(0);
                    s.setProductCategory(p.getClassifyName());
                    s.setProductCode(p.getCode());
                    s.setProductUnit(p.getUnit());
                    s.setProductType(p.getType());
                    s.setProductName(p.getName());
                    s.setProductDefinition(p.getDefinition());
                }
            }
        }
        subscribe.setSubscribeDetailList(subscribeDetailList);
        return subscribe;
    }

    @Override
    public Subscribe detailByWdly(Long id) {
        Subscribe subscribe = detail(id);
        //赋值维多利亚扩展收货仓库名
        String victoriatouristJson = subscribe.getVictoriatouristJson();
        if (ObjectUtils.isNotEmpty(victoriatouristJson)) {
            JSONObject json = JSONObject.parseObject(victoriatouristJson);
            Long receiptWarehouseId = json.getLong("receiptWarehouseId");
            Warehouse warehouse = warehouseService.getById(receiptWarehouseId);
            if (ObjectUtils.isNotEmpty(victoriatouristJson)) {
                json.put("receiptWarehouseName", warehouse.getName());
                subscribe.setVictoriatouristJson(json.toJSONString());
            }
        }
        return subscribe;
    }

    @DSTransactional
    @Override
    public void add(Subscribe subscribe) {
        SubscribeService subscribeService = SpringUtil.getBean(SubscribeService.class);
        SubscribeDetailService subscribeDetailService = SpringUtil.getBean(SubscribeDetailService.class);
//        subscribe.setCode(CodeEnum.SUBSCRIBE.getCode());
        subscribe.setCode(codingRuleService.createCode(CodingRuleEnum.SUBSCRIBE.getKey(),null));
        subscribeService.save(subscribe);
        List<SubscribeDetail> SubscribeDetails = subscribe.getSubscribeDetailList();
        if (CollectionUtils.isNotEmpty(SubscribeDetails)) {
            for (SubscribeDetail s : SubscribeDetails) {
                s.setSubscribeId(subscribe.getId());
            }
            subscribeDetailService.saveBatch(subscribe.getSubscribeDetailList());
        }
    }

    /**
     * 编辑
     *
     * @param subscribeDto
     */
    @DSTransactional
    @Override
    public void edit(SubscribeDto subscribeDto) {
        this.updateById(subscribeDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<SysDept> getDepts() {
        List<Long> deptIds = new ArrayList<>();
        List<SubscribeDetail> list = subscribeDetailService.list();
        if(ObjectUtils.isEmpty(list)){
            return new ArrayList<>();
        }
        List<Long> ids = list.stream().map(SubscribeDetail::getBussinessId).collect(Collectors.toList());
        List<ProductInfo> list1 = productInfoService.listByIds(ids);
        for (ProductInfo productInfo : list1) {
            String victoriatouristJson = productInfo.getVictoriatouristJson();
            JSONObject json = ObjectUtils.isNotEmpty(victoriatouristJson) ? JSONObject.parseObject(victoriatouristJson) : new JSONObject();
            Long deptId = json.getLong("deptId");
            deptIds.add(deptId);
        }
        if(ObjectUtils.isEmpty(deptIds)){
            return new ArrayList<>();
        }
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        List<SysDept> sysDepts = deptService.listByIds(deptIds);
        DynamicDataSourceContextHolder.poll();
        return sysDepts;
    }

}
