package com.fjhx.wms.service.stock.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.wms.entity.arrival.po.ArrivalStockRecords;
import com.fjhx.wms.entity.arrival.po.ArrivalStockRecordsDetails;
import com.fjhx.wms.entity.stock.dto.StockWaitDetailsDto;
import com.fjhx.wms.entity.stock.dto.StockWaitDto;
import com.fjhx.wms.entity.stock.dto.StockWaitSelectDto;
import com.fjhx.wms.entity.stock.emums.StockWaitType;
import com.fjhx.wms.entity.stock.po.*;
import com.fjhx.wms.entity.stock.vo.StockWaitDetailsVo;
import com.fjhx.wms.entity.stock.vo.StockWaitVo;
import com.fjhx.wms.entity.warehouse.po.Warehouse;
import com.fjhx.wms.mapper.stock.StockWaitMapper;
import com.fjhx.wms.service.arrival.ArrivalStockRecordsDetailsService;
import com.fjhx.wms.service.arrival.ArrivalStockRecordsService;
import com.fjhx.wms.service.stock.*;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.fjhx.wms.utils.CodeEnum;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 待出入库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-22
 */
@DS(SourceConstant.WMS)
@Service
public class StockWaitServiceImpl extends ServiceImpl<StockWaitMapper, StockWait> implements StockWaitService {
    @Autowired
    ProductInfoService productInfoService;
    @Autowired
    StockService stockService;
    @Autowired
    StockJournalService stockJournalService;
    @Autowired
    StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    StockWaitDetailsService stockWaitDetailsService;
    @Autowired
    WarehouseService warehouseService;
    @Autowired
    ArrivalStockRecordsService arrivalStockRecordsService;
    @Autowired
    ArrivalStockRecordsDetailsService arrivalStockRecordsDetailsService;

    @Override
    public Page<StockWaitVo> getPage(StockWaitSelectDto dto) {
        IWrapper<StockWait> wrapper = getWrapper();
        wrapper.orderByDesc("sw", StockWait::getId);
        //过滤入库/出库
        wrapper.eq(StockWait::getType, dto.getType());
        wrapper.eq(StockWait::getBusinessCode, dto.getKeyword());
        Page<StockWaitVo> page = this.baseMapper.getPage1(dto.getPage(), wrapper);

        List<StockWaitVo> records = page.getRecords();
        // 赋值产品属性
        productInfoService.attributeAssign(records, StockWaitVo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCode());
            item.setProductUnit(productInfo.getUnit());
            item.setProductType(productInfo.getType());
            item.setProductName(productInfo.getName());
            item.setProductSpec(productInfo.getSpec());
        });

        return page;
    }

    @Override
    public Page<StockWaitVo> getPageByWdly(StockWaitSelectDto dto) {
        IWrapper<StockWait> wrapper = getWrapper();
        wrapper.orderByDesc("sw", StockWait::getId);
        //过滤入库/出库
        wrapper.eq(StockWait::getType, dto.getType());
        wrapper.eq(StockWait::getBusinessCode, dto.getKeyword());
        wrapper.eq(StockWait::getStatus, dto.getStatus());
        Page<StockWaitVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<StockWaitVo> records = page.getRecords();
        //赋值操作人
        UserUtil.assignmentNickName(records, StockWaitVo::getCreateUser, StockWaitVo::setOperatorName);
        //赋值仓库名称和id
        List<Long> wids = new ArrayList<>();
        for (StockWaitVo stockWaitVo : records) {
            String victoriatouristJson = stockWaitVo.getVictoriatouristJson();
            if (ObjectUtil.isEmpty(victoriatouristJson)) {
                continue;
            }
            JSONObject json = JSONObject.parseObject(victoriatouristJson);
            if (!json.containsKey("receiptWarehouseId")) {
                continue;
            }
            wids.add(json.getLong("receiptWarehouseId"));
        }
        if (ObjectUtil.isEmpty(wids)) {
            return page;
        }
        List<Warehouse> warehouses = warehouseService.listByIds(wids);
        Map<Long, String> warehousesMap = warehouses.stream().collect(Collectors.toMap(Warehouse::getId, Warehouse::getName));
        for (StockWaitVo stockWaitVo : records) {
            String victoriatouristJson = stockWaitVo.getVictoriatouristJson();
            if (ObjectUtil.isEmpty(victoriatouristJson)) {
                continue;
            }
            JSONObject json = JSONObject.parseObject(victoriatouristJson);
            json.put("receiptWarehouseName", warehousesMap.get(json.getLong("receiptWarehouseId")));
            stockWaitVo.setVictoriatouristJson(json.toJSONString());
        }

        return page;
    }

    @Override
    public StockWaitVo detail(Long id) {
        StockWait stockWait = this.getById(id);
        StockWaitVo result = BeanUtil.toBean(stockWait, StockWaitVo.class);

        StockWaitDetails stockWaitDetails = stockWaitDetailsService.getOne(q -> q.eq(StockWaitDetails::getStockWaitId, id));
        ProductInfo byId = productInfoService.getById(stockWaitDetails.getProductId());
        result.setProductName(byId.getName());
        result.setQuantity(stockWaitDetails.getQuantity());
        result.setStockWaitDetailsId(stockWaitDetails.getId());

        //获取明细列表
        List<StockWaitDetails> stockWaitDetailsList = stockWaitDetailsService.list(q -> q.eq(StockWaitDetails::getStockWaitId, id));
        List<StockWaitDetailsVo> stockWaitDetailsVos = BeanUtil.copyToList(stockWaitDetailsList, StockWaitDetailsVo.class);
        //赋值产品信息
        productInfoService.attributeAssign(stockWaitDetailsVos, StockWaitDetailsVo::getProductId, (item, productInfo) -> {
            item.setProductName(productInfo.getName());
        });
        result.setStockWaitDetailsList(stockWaitDetailsVos);

        return result;
    }

    @Override
    public StockWaitVo detailByWdly(Long id) {
        StockWait stockWait = this.getById(id);
        StockWaitVo result = BeanUtil.toBean(stockWait, StockWaitVo.class);
        List<StockWaitDetails> list = stockWaitDetailsService.list(q -> q.eq(StockWaitDetails::getStockWaitId, id));
        List<StockWaitDetailsVo> stockWaitDetailsVos = BeanUtil.copyToList(list, StockWaitDetailsVo.class);
        //根据业务id(到货id)查询采购单明细信息


        //赋值物品信息和采购信息
        List<Long> pids = stockWaitDetailsVos.stream().map(StockWaitDetails::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(pids)) {
            List<ProductInfo> productInfos = productInfoService.listByIds(pids);
            Map<Long, ProductInfo> productInfoMap = productInfos.stream().collect(Collectors.toMap(ProductInfo::getId, Function.identity()));
            for (StockWaitDetailsVo stockWaitDetails : stockWaitDetailsVos) {
                ProductInfo productInfo = productInfoMap.get(stockWaitDetails.getProductId());
                if (ObjectUtil.isEmpty(productInfo)) {
                    continue;
                }
                stockWaitDetails.setProductName(productInfo.getName());
                stockWaitDetails.setProductCode(productInfo.getCode());
                stockWaitDetails.setProductCustomCode(productInfo.getCustomCode());
            }
        }
        result.setStockWaitDetailsList(stockWaitDetailsVos);
        return result;
    }

    @DSTransactional
    @Override
    public void add(StockWaitDetailsDto stockWaitDetailsDto) {
        stockWaitDetailsService.add(stockWaitDetailsDto);
    }

    @DSTransactional
    @Override
    public void addByWdly(StockWaitDto stockWaitDto) {
        List<Stock> stockList = new ArrayList<>();

        List<StockWaitDetails> stockWaitDetailsList = stockWaitDto.getStockWaitDetailsList();
        List<Long> dids = stockWaitDetailsList.stream().map(StockWaitDetails::getId).collect(Collectors.toList());
        Map<Long, StockWaitDetails> stockWaitDetailsMap = stockWaitDetailsList.stream().collect(Collectors.toMap(StockWaitDetails::getId, Function.identity()));
        List<StockWaitDetails> stockWaitDetails = stockWaitDetailsService.listByIds(dids);

        //已经完全入库数量统计
        Integer statusFlag = 0;
        for (StockWaitDetails stockWaitDetail : stockWaitDetails) {
            StockWaitDetails temp = stockWaitDetailsMap.get(stockWaitDetail.getId());
            //更新已入库数量
            BigDecimal receiptQuantity = stockWaitDetail.getReceiptQuantity() == null ? BigDecimal.ZERO : stockWaitDetail.getReceiptQuantity();
            stockWaitDetail.setReceiptQuantity(receiptQuantity.add(temp.getQuantity()));
            if (stockWaitDetail.getReceiptQuantity().compareTo(stockWaitDetail.getQuantity()) > 0) {
                throw new ServiceException("入库数量+已入库数量不能大于待采购数量");
            }

            //计算已经完全入库的数量
            if (stockWaitDetail.getReceiptQuantity().compareTo(stockWaitDetail.getQuantity()) == 0) {
                statusFlag++;
            }
        }

        //操作库存
        for (StockWaitDetails waitDetails : stockWaitDetailsList) {
            Stock stock = new Stock();
            stock.setQuantity(waitDetails.getQuantity());
            stock.setProductId(waitDetails.getProductId());
            stockList.add(stock);
        }

        stockWaitDetailsService.updateBatchById(stockWaitDetails);
        //修改待入库状态
        StockWait stockWait = getById(stockWaitDto.getId());
        if (statusFlag == stockWaitDetails.size()) {
            stockWait.setStatus(2);//入库完成
        } else {
            stockWait.setStatus(1);//部分入库
        }
        String victoriatouristJson = stockWait.getVictoriatouristJson();
        JSONObject json = ObjectUtil.isNotEmpty(victoriatouristJson) ? JSONObject.parseObject(victoriatouristJson) : new JSONObject();
        json.put("receiptWarehouseId", stockWaitDto.getWarehouseId());
        stockWait.setVictoriatouristJson(json.toJSONString());
        updateById(stockWait);

        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setOpType(stockWaitDto.getType() == 1 ? 1 : 2);
        stockJournal.setType(stockWaitDto.getType() == 1 ? 4 : 5);
        stockJournal.setCode(stockWait.getType() == 1 ? CodeEnum.SIN_CODE.getCode() : CodeEnum.SOUT_CODE.getCode());
        stockJournal.setWarehouseId(stockWaitDto.getWarehouseId());
        stockJournal.setBusinessId(stockWaitDto.getId());
        //保存出入库记录
        stockJournalService.save(stockJournal);

        List<StockJournalDetails> stockJournalDetailsList = stockService.ModifyInventory(stockJournal.getId(), 3, stockList, stockWaitDto.getWarehouseId());
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);

        //如果是采购到货 创建待质检记录
        if (stockWait.getBusinessType() == StockWaitType.PURCHASE_ARRIVAL_IN.getDetailType()) {
            //创建采购到货入库记录
            ArrivalStockRecords arrivalStockRecords = new ArrivalStockRecords();
            arrivalStockRecords.setStockWaitId(stockWait.getId());
            arrivalStockRecords.setWarehouseId(stockWaitDto.getWarehouseId());
            arrivalStockRecords.setQualityStatus(0);
            arrivalStockRecords.setArrivalCode(json.getString("arrivalCode"));//设置到货编码
            arrivalStockRecordsService.save(arrivalStockRecords);
            //创建入库明细
            List<ArrivalStockRecordsDetails> arrivalStockRecordsDetailsList = new ArrayList<>();
            for (StockWaitDetails waitDetails : stockWaitDetailsList) {
                ArrivalStockRecordsDetails arrivalStockRecordsDetails = new ArrivalStockRecordsDetails();
                arrivalStockRecordsDetails.setArrivalStockRecordsId(arrivalStockRecords.getId());
                arrivalStockRecordsDetails.setProductInfoId(waitDetails.getProductId());
                arrivalStockRecordsDetails.setQuantity(waitDetails.getQuantity());
                arrivalStockRecordsDetails.setQualityQuantity(BigDecimal.ZERO);
                arrivalStockRecordsDetailsList.add(arrivalStockRecordsDetails);
            }
            arrivalStockRecordsDetailsService.saveBatch(arrivalStockRecordsDetailsList);
        }
    }

    @Override
    public void edit(StockWaitDto stockWaitDto) {
        this.updateById(stockWaitDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
